;;; generate.cl --- Declt reference manual generation script

;; Copyright (C) 2010-2013 Didier Verna

;; Author: Didier Verna <didier@didierverna.net>

;; This file is part of Declt.

;; Declt is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License version 3,
;; as published by the Free Software Foundation.

;; Declt is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;; Commentary:



;;; Code:

(require :asdf)
(asdf:load-system :asdf)

(defconstant +introduction+
  "@macro declt
@t{Declt}
@end macro

@macro cl
Common-Lisp
@end macro

@macro etc
@i{etc.}
@end macro

@declt{} (pronounce ``dec'let'') is a reference manual generator for
@cl{} libraries.  It works by loading an ASDF system and introspecting
its contents. The generated documentation contains the description for
the system itself and its components (modules and files), the packages
defined in that system and the definitions found in those packages.

Exported and internal definitions are listed separately. This allows the
reader to have a quick view on the library's public API. Within each
section, definitions are sorted lexicographically.

In addition to ASDF system components and packages, @declt{} documents
the following definitions: constants, special variables, symbol macros,
macros, @t{setf} expanders, compiler macros, functions (including
@t{setf} ones), generic functions and methods (including @t{setf} ones),
method combinations, conditions, structures, classes and types.

The generated documentation includes every possible bit of information
that introspecting can provide: documentation strings, lambda lists
(including qualifiers and specializers where appropriate), slots
(including type, allocation and initialization arguments), definition
source file @etc{}

Every documented item provides a full set of cross-references to related
items: ASDF component dependencies, parents and children, classes direct
methods, super and subclasses, slot readers and writers, @t{setf}
expanders access and update functions @etc{}

Finally, @declt{} produces exhaustive and multiple-entry indexes for
every documented item.

Reference manuals are generated in Texinfo format (compatible, but not
requiring Texinfo 5). From there it is possible to produce readable /
printable outut in info, HTML, PDF, DVI and PostScript with tools such as
@command{makeinfo}, @command{texi2dvi} or @command{texi2pdf}.

The @declt{} Reference Manual, which you are currently reading,
is the primary example of documentation generated by @declt{} itself.
@ifinfo
@xref{Top, , , declt-user, The Declt User Manual},
@end ifinfo
@ifnotinfo
@xref{Top, , , user, The Declt User Manual},
@end ifnotinfo
for a more human-readable guide to using @declt{}."
  "The reference manual's introductory text.")

(asdf:load-system :com.dvlsoft.declt)
(com.dvlsoft.declt:nickname-package)

(if (and (second sb-ext:*posix-argv*)
	 (string= (second sb-ext:*posix-argv*) "--web"))
    (declt:declt :com.dvlsoft.declt
		 :library-name "Declt"
		 :texi-file "webreference.texi"
		 :info-file "declt-webreference" ; but we don't care
		 :introduction +introduction+
		 :license :gpl
		 :copyright-date "2010--2013")
    (declt:declt :com.dvlsoft.declt
		 :library-name "Declt"
		 :texi-file "reference.texi"
		 :info-file "declt-reference"
		 :introduction +introduction+
		 :license :gpl
		 :copyright-date "2010--2013"
		 :hyperlinks t))

(sb-ext:quit)


;;; generate.cl ends here
